import asyncio
import pytest
from PyQt5.QtCore import QLocale, Qt
from PyQt5.QtTest import QTest
from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QMessageBox, QApplication
from sakia.gui.sub.transfer.controller import TransferController
from duniterpy.documents import Transaction


@pytest.mark.asyncio
async def test_transfer(
    application_with_one_connection, fake_server_with_blockchain, bob, alice
):
    transfer_dialog = TransferController.create(None, application_with_one_connection)

    def close_dialog():
        if transfer_dialog.view.isVisible():
            transfer_dialog.view.hide()

    async def exec_test():
        QTest.mouseClick(transfer_dialog.view.radio_pubkey, Qt.LeftButton)
        QTest.keyClicks(transfer_dialog.view.edit_pubkey, alice.key.pubkey)
        transfer_dialog.view.spinbox_amount.setValue(10)
        await asyncio.sleep(0.1)
        assert not transfer_dialog.view.button_box.button(
            QDialogButtonBox.Ok
        ).isEnabled()
        await asyncio.sleep(0.1)
        QTest.keyClicks(transfer_dialog.view.password_input.edit_secret_key, bob.salt)
        QTest.keyClicks(transfer_dialog.view.password_input.edit_password, bob.password)
        assert transfer_dialog.view.button_box.button(QDialogButtonBox.Ok).isEnabled()
        QTest.mouseClick(
            transfer_dialog.view.button_box.button(QDialogButtonBox.Ok), Qt.LeftButton
        )
        await asyncio.sleep(0.2)
        assert isinstance(fake_server_with_blockchain.forge.pool[0], Transaction)

    application_with_one_connection.loop.call_later(10, close_dialog)
    transfer_dialog.view.show()
    await exec_test()


@pytest.mark.asyncio
async def test_transfer_chained_tx(
    application_with_two_connections, fake_server_with_blockchain, bob, john
):
    transfer_dialog = TransferController.create(None, application_with_two_connections)

    def close_dialog():
        if transfer_dialog.view.isVisible():
            transfer_dialog.view.hide()

    async def exec_test():
        QTest.mouseClick(transfer_dialog.view.radio_pubkey, Qt.LeftButton)
        QTest.keyClicks(transfer_dialog.view.edit_pubkey, john.key.pubkey)
        transfer_dialog.view.spinbox_amount.setValue(1000)
        await asyncio.sleep(0.1)
        assert not transfer_dialog.view.button_box.button(
            QDialogButtonBox.Ok
        ).isEnabled()
        await asyncio.sleep(0.1)
        QTest.keyClicks(transfer_dialog.view.password_input.edit_secret_key, bob.salt)
        QTest.keyClicks(transfer_dialog.view.password_input.edit_password, bob.password)
        assert transfer_dialog.view.button_box.button(QDialogButtonBox.Ok).isEnabled()
        QTest.mouseClick(
            transfer_dialog.view.button_box.button(QDialogButtonBox.Ok), Qt.LeftButton
        )
        await asyncio.sleep(0.2)
        assert isinstance(fake_server_with_blockchain.forge.pool[0], Transaction)
        assert fake_server_with_blockchain.forge.pool[0].comment == "[CHAINED]"
        assert isinstance(fake_server_with_blockchain.forge.pool[1], Transaction)

        application_with_two_connections.loop.call_later(10, close_dialog)

    transfer_dialog.view.show()
    await exec_test()


@pytest.mark.asyncio
async def test_transfer_from_source(
    application_with_one_connection, fake_server_with_blockchain, bob, alice
):
    def close_dialog():
        if transfer_dialog.view.isVisible():
            transfer_dialog.view.hide()

    async def exec_test():
        await asyncio.sleep(0.1)
        assert not transfer_dialog.view.button_box.button(
            QDialogButtonBox.Ok
        ).isEnabled()
        assert transfer_dialog.view.spinbox_amount.value() == 1
        await asyncio.sleep(0.1)
        # simulate checked source condition
        transfer_dialog.view.button_source_check.setDisabled(True)
        QTest.keyClicks(transfer_dialog.view.password_input.edit_secret_key, bob.salt)
        QTest.keyClicks(transfer_dialog.view.password_input.edit_password, bob.password)
        assert transfer_dialog.view.button_box.button(QDialogButtonBox.Ok).isEnabled()
        QTest.mouseClick(
            transfer_dialog.view.button_box.button(QDialogButtonBox.Ok), Qt.LeftButton
        )
        await asyncio.sleep(0.2)
        assert isinstance(fake_server_with_blockchain.forge.pool[0], Transaction)

    application_with_one_connection.loop.call_later(10, close_dialog)

    bob_connection = application_with_one_connection.db.connections_repo.get_one(
        pubkey=bob.key.pubkey
    )
    source = application_with_one_connection.sources_service.get_one(
        pubkey=bob.key.pubkey
    )
    transfer_dialog = TransferController.open_transfer_with_pubkey(
        None, application_with_one_connection, bob_connection, alice.key.pubkey, source
    )
    await exec_test()
